/**
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * Copyright (C) 2013, 2014 Department of Informatics and
 * Telecommunications, National and Kapodistrian University of Athens
 *
 *      Author: Stella Giannakopoulou <sgian@di.uoa.gr>
 */

#include "method.h"

class Consolidated_Data: public Method
{	
	private:
		Cons_Arrangement_2 arr;
		Cons_Formatter formatter;
	
	public:
		Consolidated_Data() : Method() {}
		void create_arrangement(ifstream &input_geometries);
		void insert_in_arrangement(CoordinateSequence *cords, bool is_hole, bool is_ccw, string numGeo);
		void read_arrangement();
		void write_arrangement();
		void write_blocks();
		Polygon* block_to_wkt(const Cons_Arrangement_2::Ccb_halfedge_const_circulator circ);
		void arrangement_statistics();
		bool check_validity(const Cons_Arrangement_2 &arr);
		bool is_valid(const Cons_Arrangement_2::Face_const_iterator &f);
		
		/*****Landmarks*****/
		void assign_blocks_to_landmarks(int geometries_start, int maxnodeid);
		void find_interior_exterior_blocks(string node, int maxnodeid, char *assigned_blocks, char *hole_blocks) const ;
		bool tangential_interior(const Cons_Arrangement_2::Ccb_halfedge_const_circulator circ, string node, string &is_hole, char *assigned_blocks, char *hole_blocks) const ;
		void assign_block_holes(const Cons_Arrangement_2::Face_const_handle &f, string node, string is_hole, char *assigned_blocks, char *hole_blocks) const;
		void set_hole_faces_exterior(Cons_Arrangement_2::Ccb_halfedge_const_circulator circ, string node, char *assigned_blocks, char *hole_blocks) const;
		void non_tangential_blocks(string node, char *assigned_blocks, char *hole_blocks) const;
		bool non_tangential_interior(const Cons_Arrangement_2::Ccb_halfedge_const_circulator circ, string node, char *assigned_blocks, char *hole_blocks) const;

		void find_interior_exterior_blocks_threaded(ifstream &input_geometries, int totalGeometries, int maxnodeid);
};

typedef struct cons_data_info 
{
	Consolidated_Data method;
	string node;

} thread_info;

void block_landmark_position(string node, int maxnodeid, thread_result *res);
bool same_direction(Point_2 source1, Point_2 target1, Point_2 source2, Point_2 target2);
