/**
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * Copyright (C) 2013, 2014 Department of Informatics and
 * Telecommunications, National and Kapodistrian University of Athens
 *
 *      Author: Stella Giannakopoulou <sgian@di.uoa.gr>
 */

#define MAXTRIES 10

#include "method.h"

class Interior_Point: public Method
{	
	private:
		IntPoint_Arrangement_2 arr;
		IntPoint_Formatter formatter;
	
	public:	
		list<Point_2> blockPoints;										//a list with an interior point of each block
	
	public:
		Interior_Point() : Method() {}
		void create_arrangement(ifstream &input_geometries);
		void insert_in_arrangement(CoordinateSequence *cords);
		void read_arrangement();
		void write_arrangement();
		void write_blocks();
		Polygon* block_to_wkt(const IntPoint_Arrangement_2::Ccb_halfedge_const_circulator circ);
		Point_2 get_interior_point(const IntPoint_Arrangement_2::Face_const_handle &f);
		void arrangement_statistics();
		bool check_validity(const IntPoint_Arrangement_2 &arr);
		bool is_valid(const IntPoint_Arrangement_2::Face_const_iterator &f);
		
		/*****Landmarks*****/
		void assign_blocks_to_landmarks(int geometries_start, int maxnodeid);
		void landmark_interiors(const Polygon *poly, int node);
		void landmark_interiors(const MultiPolygon *multipoly, int node);
		bool block_inside(Point_2 blockPoint, const Polygon *poly, const CoordinateSequence *exteriorCords);
		bool insideHole(const list <Point_2> &holePoints, Point_2 blockPoint);
		
		void find_interior_exterior_blocks_threaded(ifstream &input_geometries);
};

double calculate_angle(Vector v1, Vector v2);
