/**
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * Copyright (C) 2013, 2014 Department of Informatics and
 * Telecommunications, National and Kapodistrian University of Athens
 *
 *      Author: Stella Giannakopoulou <sgian@di.uoa.gr>
 */
 
#include "method.h"

// Define a functor for creating a label.
struct Overlay_label
{
	string operator() (string i, string j) const
	{
		int a = atoi(i.substr(0, 1).c_str());
		int b = atoi(j.c_str());
		int res = a+b;
		ostringstream ostr;
		ostr << res << i.substr(1);

		return ostr.str();
  }
};

typedef CGAL::Arr_face_overlay_traits<Ov_Arrangement_2, Ov_Arrangement_2, Ov_Arrangement_2, Overlay_label > Overlay_traits;

class Overlay: public Method
{	
	private:
		Ov_Arrangement_2 arr;
		Ov_Formatter formatter;
	
	public:
		Overlay() : Method() {}
		void create_arrangement(ifstream &input_geometries);
		void insert_in_arrangement(Ov_Arrangement_2 &arrang, CoordinateSequence *cords);
		void read_arrangement();
		void write_arrangement();
		void write_blocks();
		Polygon* block_to_wkt(const Ov_Arrangement_2::Ccb_halfedge_const_circulator circ);
		void arrangement_statistics();
		bool check_validity(const Ov_Arrangement_2 &arr);
		bool is_valid(const Ov_Arrangement_2::Face_const_iterator &f);
		
		/*****Landmarks*****/
		void assign_blocks_to_landmarks(int geometries_start, int maxnodeid);
};
