/**
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * Copyright (C) 2013, 2014 Department of Informatics and
 * Telecommunications, National and Kapodistrian University of Athens
 *
 *      Author: Stella Giannakopoulou <sgian@di.uoa.gr>
 */

#ifndef _QQ_CONSISTENCY_H_
#define _QQ_CONSISTENCY_H_

#include <boost/program_options.hpp>
#include "utils.h"
#include "consolidated_data.h"
#include "interior_point.h"
#include "overlay.h"

RELTYPE **renz_matrix;
char **mark;
char *node_type;
char **interior_exterior_blocks;
extern const char *input_data;
extern int srid;
string exe_dir;
string blocks_method;

/***Timer*/
double consistency_timer = 0;

/** Options **/
const char *input_data;
bool intersection = true; 				/*by default it is considered that the input geometries intersect*/
const char *blocks_file = NULL, *arrangement_output_file = NULL; 		/*the files that contain the resulting blocks or the whole arrangement*/
const char *arrangement_input_file = NULL; /*the file that constains the arrangement given as input*/
bool write_blocks_to_file = false, write_arrangement_to_file = false, read_arrangement_from_file = false;
int srid;
bool lcr = false, sd = false;
bool parallel = false, statistics = false, debug = false, print_time = false;

int backtrack_depth = 0, consistency_check = 0;
RELTYPE splitinfo[MAXSET][MAXREL+1];

/** define some variables hard-coded in Renz's code */
int swdebug; // no debug
int pcops;   // path consistency operations
int pcits;   // path consistency iterations

void check_consistency();
bool basic_consistency(RELTYPE **csp, Method &method, int maxnodeid, FILE *infile, char *csptype, int geometries_start);
bool split_relation(RELTYPE **csp, int maxnodeid, int *node1, int *node2);
void gen_split(RELTYPE r, RELTYPE *split);

#endif
