/**
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * Copyright (C) 2013, 2014 Department of Informatics and
 * Telecommunications, National and Kapodistrian University of Athens
 *
 *      Author: Stella Giannakopoulou <sgian@di.uoa.gr>
 */

#ifndef UTILS_H
#define UTILS_H

#define RELTYPE short int
#define RELTYPESIZE sizeof(RELTYPE)

#define RCC5

#ifdef __cplusplus /* If this is a C++ compiler, use C linkage */
extern "C" {
// for Renz's code
#if defined(RCC5)
#include "renz-solver/rcc5.h"
#include "renz-solver/rcc5io.h"
#include "renz-solver/rcc5op.h"
#elif defined (RCC8)
#include "renz-solver/rcc8.h"
#include "renz-solver/rcc8io.h"
#include "renz-solver/rcc8op.h"
#endif
}
#endif

#include <CGAL/Exact_predicates_exact_constructions_kernel.h>
//#include <CGAL/Filtered_kernel.h>
#include <CGAL/Polygon_2_algorithms.h>
#include <CGAL/Boolean_set_operations_2.h>
#include <CGAL/Polygon_2.h>
#include <CGAL/Constrained_Delaunay_triangulation_2.h>

#include <CGAL/Arr_segment_traits_2.h>
#include <CGAL/Arrangement_2/Arrangement_on_surface_2_global.h>
#include <CGAL/Arr_consolidated_curve_data_traits_2.h>
#include <CGAL/Arr_landmarks_point_location.h>
#include <CGAL/Arr_extended_dcel.h>
#include <CGAL/Arr_overlay_2.h>
#include <CGAL/Arr_default_overlay_traits.h>

#include <CGAL/IO/Arr_text_formatter.h>
#include <CGAL/IO/Arr_iostream.h>

#include <sys/time.h>
#include <list>
#include <fstream>
#include <cstring>
#include <math.h>
#include <sstream>

#include <geos/io/WKTReader.h>
#include <geos/geom/Geometry.h>
#include <geos/geom/Polygon.h>
#include <geos/geom/LineString.h>
#include <geos/geom/PrecisionModel.h>
#include <geos/geom/Coordinate.h>
#include <geos/geom/CoordinateArraySequence.h>
#include <geos/geom/LinearRing.h>
#include <CGAL/number_utils.h>

#include "Arr_consolidated_formatter.h"

using namespace std;
using namespace geos;
using namespace geos::io;
using namespace geos::geom;
using namespace geom;

//typedef CGAL::Simple_cartesian<double> CK;
//typedef CGAL::Filtered_kernel<CK> Kernel;
typedef CGAL::Exact_predicates_exact_constructions_kernel Kernel;
typedef Kernel::FT  FT;
typedef CGAL::Arr_segment_traits_2<Kernel> Traits_2;
typedef Traits_2::Point_2 Point_2;
typedef Traits_2::X_monotone_curve_2 Segment_2;
typedef Traits_2::Equal_2 Equal_2;
typedef CGAL::Polygon_2<Kernel> Polygon_2;
typedef CGAL::Vector_2<Kernel> Vector;

//Triangulation
typedef CGAL::Triangulation_vertex_base_2<Kernel> Vb;
typedef CGAL::Constrained_triangulation_face_base_2<Kernel> Fb;
typedef CGAL::Triangulation_data_structure_2<Vb,Fb> TDS;
typedef CGAL::Exact_predicates_tag Itag;
typedef CGAL::Constrained_Delaunay_triangulation_2<Kernel, TDS, Itag> CDT;
typedef CDT::Point CDT_Point;

//Arrangement for the interior point method
typedef CGAL::Arrangement_2<Traits_2> IntPoint_Arrangement_2;
typedef CGAL::Arr_text_formatter<IntPoint_Arrangement_2>  IntPoint_Formatter;

//Arrangement for the consolidated data method
typedef CGAL::Arr_consolidated_curve_data_traits_2 <Traits_2, string>  Data_Traits_2;
typedef Data_Traits_2::X_monotone_curve_2 Data_Segment_2;
typedef CGAL::Arr_face_extended_dcel<Data_Traits_2, int> Dcel;
typedef CGAL::Arrangement_2<Data_Traits_2, Dcel> Cons_Arrangement_2;
typedef Arr_consolidated_curve_text_formatter<Cons_Arrangement_2>  Cons_Formatter;

//Arrangement for the overlay method
typedef CGAL::Arr_face_extended_dcel<Traits_2, string> Ov_Dcel;
typedef CGAL::Arrangement_2<Traits_2, Ov_Dcel> Ov_Arrangement_2;
typedef CGAL::Arr_face_extended_text_formatter<Ov_Arrangement_2>  Ov_Formatter;

#endif /* UTILS_H */
